@@ -7,7 +7,7 @@
  *
  */
 
-/* Copyright (C) 2009, 2010, 2011 Jose E. Marchesi */
+/* Copyright (C) 2009, 2010, 2011, 2012 Jose E. Marchesi */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -85,10 +85,6 @@ struct rec_parser_s
   size_t line; /* Current line number. */
   size_t character; /* Current offset from the beginning of the file,
                        in characters.  */
-
-  bool ordered_p;   /* Sort the stuff while parsing.  */
-  char *order_rset; /* NULL => Sort all record sets.  */
-  rec_field_name_t order_by_field;
 };
 
 const char *rec_parser_error_strings[] =
@@ -105,6 +101,15 @@ const char *rec_parser_error_strings[] =
   NULL /* Sentinel */
 };
 
+/* The following macro is used by some functions to reduce
+   verbosity.  */
+
+#define FNAME(id) rec_std_field_name ((id))
+
+/*
+ * Public functions.
+ */
+
 rec_parser_t
 rec_parser_new (FILE *in,
                 char *source)
@@ -152,12 +157,6 @@ rec_parser_new_str (char *buffer,
 void
 rec_parser_destroy (rec_parser_t parser)
 {
-  if (parser->order_by_field)
-    {
-      rec_field_name_destroy (parser->order_by_field);
-    } 
-
-  free (parser->order_rset);
   free (parser->source);
   free (parser);
 }
@@ -388,7 +387,7 @@ rec_parse_record (rec_parser_t parser,
   if (rec_parse_field (parser, &field))
     {
       /* Add the field to the record */
-      rec_record_append_field (new, field);
+      rec_mset_append (rec_record_mset (new), MSET_FIELD, (void *) field, MSET_ANY);
     }
   else
     {
@@ -408,7 +407,7 @@ rec_parse_record (rec_parser_t parser,
           if (rec_parse_comment (parser, &comment))
             {
               /* Add the comment to the record.  */
-              rec_record_append_comment (new, comment);
+              rec_mset_append (rec_record_mset (new), MSET_COMMENT, (void *) comment, MSET_ANY);
             }
         }
       else if (c == '\n')
@@ -423,7 +422,7 @@ rec_parse_record (rec_parser_t parser,
           if (rec_parse_field (parser, &field))
             {
               /* Add the field to the record */
-              rec_record_append_field (new, field);
+              rec_mset_append (rec_record_mset (new), MSET_FIELD, (void *) field, MSET_ANY);
             }
           else
             {
@@ -457,51 +456,28 @@ rec_parse_rset (rec_parser_t parser,
   char c;
   rec_rset_t new;
   rec_record_t record;
-  rec_field_name_t rec_fname;
   rec_comment_t comment;
   size_t comments_added = 0;
 
   ret = false;
 
-  if ((rec_fname = rec_field_name_new ()) == NULL)
-    {
-      /* Out of memory */
-      parser->error = REC_PARSER_ENOMEM;
-      return false;
-    }
-  
-  rec_field_name_set (rec_fname, 0, "%rec");
-
   if ((new = rec_rset_new ()) == NULL)
     {
       /* Out of memory */
       parser->error = REC_PARSER_ENOMEM;
-      rec_field_name_destroy (rec_fname);
       return false;
     }
 
-  /* Set the ordered attribute of the rset to the desired value.  */
-  rec_rset_set_ordered (new, parser->ordered_p);
-
   /* Set the descriptor for this record set.  */
   rec_rset_set_descriptor (new, parser->prev_descriptor);
   parser->prev_descriptor = NULL;
 
-  /* Change the sorting criteria if requested.  */
-  if (parser->order_by_field
-      && (!parser->order_rset
-          || (strcmp (parser->order_rset,
-                      rec_rset_type (new)) == 0)))
-    {
-      rec_rset_set_order_by_field (new, parser->order_by_field);
-    }
-
   while ((ci = rec_parser_getc (parser)) != EOF)
     {
       c = (char) ci;
 
       /* Skip newline characters and blanks.  */
-      if ((c == '\n') || (c == ' '))
+      if ((c == '\n') || (c == ' ') || (c == '\t'))
         {
           continue;
         }
@@ -512,7 +488,7 @@ rec_parse_rset (rec_parser_t parser,
           rec_parse_comment (parser, &comment);
 
           /* Add the comment to the record set.  */
-          rec_rset_append_comment (new, comment);
+          rec_mset_append (rec_rset_mset (new), MSET_COMMENT, (void *) comment, MSET_ANY);
 
           comments_added++;
         }
@@ -531,8 +507,7 @@ rec_parse_rset (rec_parser_t parser,
 
                  Otherwise, add the record to the current record
                  set. */
-              if (rec_record_field_p (record,
-                                      rec_fname))
+              if (rec_record_field_p (record, FNAME(REC_FIELD_REC)))
                 {
                   if ((rec_rset_num_records (new) == 0) &&
                       (!rec_rset_descriptor (new)))
@@ -541,15 +516,6 @@ rec_parse_rset (rec_parser_t parser,
                          input stream is a descriptor. */
                       rec_rset_set_descriptor (new, record);
                       rec_rset_set_descriptor_pos (new, comments_added);
-
-                      /* Change the sorting criteria if requested.  */
-                      if (parser->order_by_field
-                          && (!parser->order_rset
-                              || (strcmp (parser->order_rset,
-                                          rec_rset_type (new)) == 0)))
-                        {
-                          rec_rset_set_order_by_field (new, parser->order_by_field);
-                        }
                     }
                   else
                     {
@@ -560,7 +526,8 @@ rec_parse_rset (rec_parser_t parser,
                 }
               else
                 {
-                  rec_rset_append_record (new, record);
+                  rec_record_set_container (record, new);
+                  rec_mset_append (rec_rset_mset (new), MSET_RECORD, (void *) record, MSET_ANY);
                 }
             }
           else
@@ -703,41 +670,6 @@ rec_parse_record_str (char *str)
   return record;
 }
 
-void
-rec_parser_set_ordered (rec_parser_t parser,
-                        bool ordered_p)
-{
-  parser->ordered_p = ordered_p;
-}
-
-bool
-rec_parser_ordered (rec_parser_t parser)
-{
-  return parser->ordered_p;
-}
-
-void
-rec_parser_sort_rset (rec_parser_t parser,
-                      char *rset_name,
-                      rec_field_name_t field_name)
-{
-  if (field_name)
-    {
-      if (parser->order_by_field)
-        {
-          rec_field_name_destroy (parser->order_by_field);
-        }
-      free (parser->order_rset);
-
-      parser->order_by_field = rec_field_name_dup (field_name);
-      
-      if (rset_name)
-        {
-          parser->order_rset = strdup (rset_name);
-        }
-    }
-}
-
 /*
  * Private functions
  */
@@ -1280,9 +1212,6 @@ rec_parser_init_common (rec_parser_t parser,
   parser->character = 0;
   parser->prev_descriptor = NULL;
   parser->p = parser->in_buffer;
-  parser->ordered_p = false;
-  parser->order_by_field = NULL;
-  parser->order_rset = NULL;
 
   return true;
 }
