@@ -7,7 +7,7 @@
  *
  */
 
-/* Copyright (C) 2009, 2010, 2011 Jose E. Marchesi */
+/* Copyright (C) 2009, 2010, 2011, 2012 Jose E. Marchesi */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -27,10 +27,11 @@
 
 #include <stdlib.h>
 #include <stdint.h>
+#include <errno.h>
+#include <locale.h>
 #include <string.h>
 #include <parse-datetime.h>
 
-#include <rec-mset.h>
 #include <rec.h>
 #include <rec-utils.h>
 
@@ -80,9 +81,8 @@ struct rec_rset_s
   /* Type registry.  */
   rec_type_reg_t type_reg;
 
-  /* Field to order by.  */
+  /* Field to order by specified in the record descriptor.  */
   rec_field_name_t order_by_field;
-  bool ordered_p;
 
   /* Size constraints.  */
   size_t min_size;
@@ -94,37 +94,6 @@ struct rec_rset_s
   rec_mset_t mset;
 };
 
-/* The 'record_type' in the mset contains a list of pointers to
-   rec_rset_record_s structures, instead of pointers to rec_record_t.
-   This is done in this way to allow the callbacks to access to
-   properties of the containing record set by using the 'rset'
-   field.  */
-
-struct rec_rset_record_s
-{
-  rec_rset_t   rset;
-  rec_record_t record;
-};
-
-typedef struct rec_rset_record_s *rec_rset_record_t;
-
-/* Same for comment_type and rec_rset_comment_s.  */
-
-struct rec_rset_comment_s
-{
-  rec_rset_t rset;
-  rec_comment_t comment;
-};
-
-typedef struct rec_rset_comment_s *rec_rset_comment_t;
-
-/* Set of names for special fields */
-
-#define REC_NAME_REC "%rec"
-#define REC_NAME_KEY "%key"
-#define REC_NAME_MANDATORY "%mandatory"
-#define REC_NAME_UNIQUE "%unique:"
-
 /* Static functions implemented below.  */
 
 static void rec_rset_update_types (rec_rset_t rset);
@@ -149,6 +118,18 @@ static rec_rset_fprops_t rec_rset_get_props (rec_rset_t rset,
                                              rec_field_name_t fname,
                                              bool create_p);
 
+static void rec_rset_add_auto_field_int (rec_rset_t rset,
+                                         rec_field_name_t field_name,
+                                         rec_record_t record);
+static void rec_rset_add_auto_field_date (rec_rset_t rset,
+                                          rec_field_name_t field_name,
+                                          rec_record_t record);
+
+/* The following macro is used by some functions to reduce
+   verbosity.  */
+
+#define FNAME(id) rec_std_field_name ((id))
+
 /*
  * Public functions.
  */
@@ -179,9 +160,10 @@ rec_rset_new (void)
 
           /* No order by field, initially.  */
           rset->order_by_field = NULL;
-          rset->ordered_p = false;
 
-          /* register the types.  */
+          /* register the types.  See rec.h for the definition of
+             MSET_COMMENT and MSET_RECORD.  */
+
           rset->record_type = rec_mset_register_type (rset->mset,
                                                       "record",
                                                       rec_rset_record_disp_fn,
@@ -244,14 +226,14 @@ rec_rset_destroy (rec_rset_t rset)
 rec_rset_t
 rec_rset_dup (rec_rset_t rset)
 {
-  rec_rset_t new;
+  rec_rset_t new = NULL;
 
   new = malloc (sizeof (struct rec_rset_s));
   if (new)
     {
       new->record_type = rset->record_type;
       new->comment_type = rset->comment_type;
-      new->mset = rec_mset_dup (rset->mset);
+      new->mset = NULL;
       new->min_size = rset->min_size;
       new->max_size = rset->max_size;
       /* XXX: make copies of the following structures.  */
@@ -263,349 +245,35 @@ rec_rset_dup (rec_rset_t rset)
           new->order_by_field =
             rec_field_name_dup (rset->order_by_field);
         }
-      new->ordered_p = rset->ordered_p;
     }
 
+  new->mset = rec_mset_dup (rset->mset);
+  
   return new;
 }
 
-int
+rec_mset_t
+rec_rset_mset (rec_rset_t rset)
+{
+  return rset->mset;
+}
+
+size_t
 rec_rset_num_elems (rec_rset_t rset)
 {
-  return rec_mset_count (rset->mset,
-                         MSET_ANY);
+  return rec_mset_count (rset->mset, MSET_ANY);
 }
 
-int
+size_t
 rec_rset_num_records (rec_rset_t rset)
 {
-  return rec_mset_count (rset->mset,
-                         rset->record_type);
+  return rec_mset_count (rset->mset, rset->record_type);
 }
 
-int
+size_t
 rec_rset_num_comments (rec_rset_t rset)
 {
-  return rec_mset_count (rset->mset,
-                         rset->comment_type);
-}
-
-rec_rset_elem_t
-rec_rset_null_elem (void)
-{
-  rec_rset_elem_t elem;
-
-  elem.mset_elem = NULL;
-
-  return elem;
-}
-
-rec_rset_elem_t
-rec_rset_get_elem (rec_rset_t rset,
-                   int position)
-{
-  rec_rset_elem_t elem;
-
-  elem.mset_elem = rec_mset_get (rset->mset,
-                                 MSET_ANY,
-                                 position);
-
-  return elem;
-}
-
-rec_rset_elem_t
-rec_rset_get_record (rec_rset_t rset,
-                     int position)
-{
-  rec_rset_elem_t elem;
-
-  elem.mset_elem = rec_mset_get (rset->mset,
-                                 rset->record_type,
-                                 position);
-
-  return elem;
-}
-
-rec_rset_elem_t
-rec_rset_get_comment (rec_rset_t rset,
-                      int position)
-{
-  rec_rset_elem_t elem;
-
-  elem.mset_elem = rec_mset_get (rset->mset,
-                                 rset->comment_type,
-                                 position);
-
-  return elem;
-}
-
-bool
-rec_rset_remove_at (rec_rset_t rset,
-                    int position)
-{
-  return rec_mset_remove_at (rset->mset, position);
-}
-
-void
-rec_rset_insert_at (rec_rset_t rset,
-                    rec_rset_elem_t elem,
-                    int position)
-{
-  if (rset->ordered_p
-      && rset->order_by_field
-      && rec_rset_elem_record_p (rset, elem))
-    {
-      /* Don't insert the record at the requested position: use a
-         sorting criteria instead.  */
-      rec_mset_add_sorted (rset->mset, elem.mset_elem);
-    }
-  else
-    {
-      /* Insert at the requested position.  */
-      rec_mset_insert_at (rset->mset,
-                          elem.mset_elem,
-                          position);
-    }
-}
-
-void
-rec_rset_append (rec_rset_t rset,
-                 rec_rset_elem_t elem)
-{
-  if (rset->ordered_p
-      && rset->order_by_field
-      && rec_rset_elem_record_p (rset, elem))
-    {
-      /* Don't append the record: use a sorting criteria instead.  */
-      rec_mset_add_sorted (rset->mset, elem.mset_elem);
-    }
-  else
-    {
-      rec_mset_append (rset->mset, elem.mset_elem);
-    }
-}
-
-void
-rec_rset_append_record (rec_rset_t rset,
-                        rec_record_t record)
-{
-  rec_rset_elem_t elem;
-  
-  elem = rec_rset_elem_record_new (rset, record);
-
-  if (rset->ordered_p && rset->order_by_field)
-    {
-      rec_mset_add_sorted (rset->mset, elem.mset_elem);
-    }
-  else
-    {
-      rec_mset_append (rset->mset, elem.mset_elem);
-    }
-}
-
-void
-rec_rset_append_comment (rec_rset_t rset,
-                         rec_comment_t comment)
-{
-  rec_rset_elem_t elem;
-
-  elem = rec_rset_elem_comment_new (rset, comment);
-  rec_mset_append (rset->mset, elem.mset_elem);
-}
-
-rec_rset_elem_t
-rec_rset_remove (rec_rset_t rset,
-                 rec_rset_elem_t elem)
-{
-  elem.mset_elem = rec_mset_remove (rset->mset, elem.mset_elem);
-  return elem;
-}
-
-rec_rset_elem_t
-rec_rset_remove_record (rec_rset_t rset,
-                        rec_rset_elem_t elem)
-{
-  elem = rec_rset_remove (rset, elem);
-  if (rec_rset_elem_p (elem)
-      && !rec_rset_elem_record_p (rset, elem))
-    {
-      elem = rec_rset_next_record (rset, elem);
-    }
-
-  return elem;
-}
-
-rec_rset_elem_t
-rec_rset_remove_comment (rec_rset_t rset,
-                         rec_rset_elem_t elem)
-{
-  elem = rec_rset_remove (rset, elem);
-  if (rec_rset_elem_p (elem)
-      && !rec_rset_elem_comment_p (rset, elem))
-    {
-      elem = rec_rset_next_comment (rset, elem);
-    }
-
-  return elem;
-}
-
-void
-rec_rset_insert_after (rec_rset_t rset,
-                       rec_rset_elem_t elem,
-                       rec_rset_elem_t new_elem)
-{
-  if (rset->ordered_p
-      && rset->order_by_field
-      && rec_rset_elem_record_p (rset, new_elem))
-    {
-      /* Don't insert the record at the specified location: use a
-         sorting criteria instead.  */
-      rec_mset_add_sorted (rset->mset, new_elem.mset_elem);
-    }
-  else
-    {
-      rec_mset_insert_after (rset->mset,
-                             elem.mset_elem,
-                             new_elem.mset_elem);
-    }
-}
-
-rec_rset_elem_t
-rec_rset_first (rec_rset_t rset)
-{
-  rec_rset_elem_t elem;
-
-  elem.mset_elem = rec_mset_first (rset->mset, MSET_ANY);
-  return elem;
-}
-
-rec_rset_elem_t
-rec_rset_first_record (rec_rset_t rset)
-{
-  rec_rset_elem_t elem;
-
-  elem.mset_elem = rec_mset_first (rset->mset, rset->record_type);
-  return elem;
-}
-
-rec_rset_elem_t
-rec_rset_first_comment (rec_rset_t rset)
-{
-  rec_rset_elem_t elem;
-
-  elem.mset_elem = rec_mset_first (rset->mset, rset->comment_type);
-  return elem;
-}
-
-rec_rset_elem_t
-rec_rset_next (rec_rset_t rset,
-               rec_rset_elem_t elem)
-{
-  elem.mset_elem = rec_mset_next (rset->mset,
-                                  elem.mset_elem,
-                                  MSET_ANY);
-
-  return elem;
-}
-
-rec_rset_elem_t
-rec_rset_next_record (rec_rset_t rset,
-                      rec_rset_elem_t elem)
-{
-  elem.mset_elem = rec_mset_next (rset->mset,
-                                  elem.mset_elem,
-                                  rset->record_type);
-
-  return elem;
-}
-
-rec_rset_elem_t
-rec_rset_next_comment (rec_rset_t rset,
-                       rec_rset_elem_t elem)
-{
-  elem.mset_elem = rec_mset_next (rset->mset,
-                                  elem.mset_elem,
-                                  rset->comment_type);
-
-  return elem;
-}
-
-rec_rset_elem_t
-rec_rset_elem_record_new (rec_rset_t rset,
-                          rec_record_t record)
-{
-  rec_rset_elem_t elem;
-  rec_rset_record_t rset_record = NULL;
-
-  /* Create the rset_record to insert in the mset.  */
-
-  rset_record = malloc (sizeof (struct rec_rset_record_s));
-  rset_record->rset = rset;
-  rset_record->record = record;
-
-  /* Insert it in the mset.  */
-
-  elem.mset_elem = rec_mset_elem_new (rset->mset, rset->record_type);
-  rec_mset_elem_set_data (elem.mset_elem, (void *) rset_record);
-
-  return elem;
-}
-
-rec_rset_elem_t
-rec_rset_elem_comment_new (rec_rset_t rset,
-                           rec_comment_t comment)
-{
-  rec_rset_elem_t elem;
-  rec_rset_comment_t rset_comment = NULL;
-
-  /* Create the rset_comment to insert in the mset.  */
-
-  rset_comment = malloc (sizeof (struct rec_rset_comment_s));
-  rset_comment->rset = rset;
-  rset_comment->comment = comment;
-
-  /* Insert it in the mset.  */
-
-  elem.mset_elem = rec_mset_elem_new (rset->mset, rset->comment_type);
-  rec_mset_elem_set_data (elem.mset_elem, (void *) rset_comment);
-  
-  return elem;
-}
-
-bool
-rec_rset_elem_p (rec_rset_elem_t elem)
-{
-  return (elem.mset_elem != NULL);
-}
-
-bool
-rec_rset_elem_record_p (rec_rset_t rset,
-                        rec_rset_elem_t elem)
-{
-  return (rec_mset_elem_type (elem.mset_elem) == rset->record_type);
-}
-
-bool
-rec_rset_elem_comment_p (rec_rset_t rset,
-                         rec_rset_elem_t elem)
-{
-  return (rec_mset_elem_type (elem.mset_elem) == rset->comment_type);
-}
-
-rec_record_t
-rec_rset_elem_record (rec_rset_elem_t elem)
-{
-  rec_rset_record_t rset_record =
-    (rec_rset_record_t) rec_mset_elem_data (elem.mset_elem);
-  return rset_record->record;
-}
-
-rec_comment_t
-rec_rset_elem_comment (rec_rset_elem_t elem)
-{
-  rec_rset_comment_t rset_comment =
-    (rec_rset_comment_t) rec_mset_elem_data (elem.mset_elem);
-  return rset_comment->comment;
+  return rec_mset_count (rset->mset, rset->comment_type);
 }
 
 rec_record_t
@@ -648,7 +316,6 @@ rec_rset_set_type (rec_rset_t rset,
                    char *type)
 {
   rec_field_t rec_field;
-  rec_field_name_t rec_field_name;
 
   if (!type)
     {
@@ -663,9 +330,8 @@ rec_rset_set_type (rec_rset_t rset,
       
     }
 
-  rec_field_name = rec_parse_field_name_str ("%rec:");
   rec_field = rec_record_get_field_by_name (rset->descriptor,
-                                            rec_field_name,
+                                            FNAME(REC_FIELD_REC),
                                             0);
 
   if (rec_field)
@@ -674,9 +340,9 @@ rec_rset_set_type (rec_rset_t rset,
     }
   else
     {
-      rec_field = rec_field_new (rec_field_name,
+      rec_field = rec_field_new (rec_field_name_dup (FNAME(REC_FIELD_REC)),
                                  type);
-      rec_record_append_field (rset->descriptor, rec_field);
+      rec_mset_append (rec_record_mset (rset->descriptor), MSET_FIELD, (void *) rec_field, MSET_FIELD);
     }
 }
 
@@ -685,14 +351,12 @@ rec_rset_type (rec_rset_t rset)
 {
   char *res;
   rec_field_t field;
-  rec_field_name_t field_name;
 
   res = NULL;
   if (rset->descriptor)
     {
-      field_name = rec_parse_field_name_str ("%rec:");
       field = rec_record_get_field_by_name (rset->descriptor,
-                                            field_name,
+                                            FNAME(REC_FIELD_REC),
                                             0);
       if (field)
         {
@@ -708,14 +372,12 @@ rec_rset_url (rec_rset_t rset)
 {
   char *res;
   rec_field_t field;
-  rec_field_name_t field_name;
 
   res = NULL;
   if (rset->descriptor)
     {
-      field_name = rec_parse_field_name_str ("%rec:");
       field = rec_record_get_field_by_name (rset->descriptor,
-                                            field_name,
+                                            FNAME(REC_FIELD_REC),
                                             0);
       if (field)
         {
@@ -737,8 +399,7 @@ rec_rset_rename_field (rec_rset_t rset,
                        rec_field_name_t field_name,
                        rec_field_name_t new_field_name)
 {
-  size_t i, j;
-  rec_field_t field;
+  size_t j;
   rec_record_t descriptor;
   rec_fex_t fex;
   char *fex_str;
@@ -748,34 +409,17 @@ rec_rset_rename_field (rec_rset_t rset,
   size_t result_size;
   rec_fex_elem_t fex_elem;
   rec_field_name_t fex_fname;
-  rec_field_name_t type_field_name;
-  rec_field_name_t key_field_name;
-  rec_field_name_t mandatory_field_name;
-  rec_field_name_t unique_field_name;
-  rec_field_name_t prohibit_field_name;
-  rec_field_name_t sort_field_name;
-#if defined REC_CRYPT_SUPPORT
-  rec_field_name_t confidential_field_name;
-#endif
 
-  type_field_name = rec_parse_field_name_str ("%type:");
-  key_field_name = rec_parse_field_name_str ("%key:");
-  mandatory_field_name = rec_parse_field_name_str ("%mandatory:");
-  unique_field_name = rec_parse_field_name_str ("%unique:");
-  prohibit_field_name = rec_parse_field_name_str ("%prohibit:");
-  sort_field_name = rec_parse_field_name_str ("%sort:");
-#if defined REC_CRYPT_SUPPORT
-  confidential_field_name = rec_parse_field_name_str ("%confidential:");
-#endif
-  
   descriptor = rec_rset_descriptor (rset);
   if (descriptor)
     {
-      for (i = 0; i < rec_record_num_fields (descriptor); i++)
+      rec_mset_t descriptor_mset = rec_record_mset (descriptor);
+      rec_mset_iterator_t iter = rec_mset_iterator (descriptor_mset);
+      rec_field_t field;
+
+      while (rec_mset_iterator_next (&iter, MSET_FIELD, (void *) &field, NULL))
         {
-          field = rec_record_elem_field (rec_record_get_field (descriptor, i));
-          
-          if (rec_field_name_eql_p (rec_field_name (field), type_field_name))
+          if (rec_field_name_eql_p (rec_field_name (field), FNAME(REC_FIELD_TYPE)))
             {
               /* Process a %type entry.  Invalid entries are
                  skipped.  */
@@ -814,14 +458,14 @@ rec_rset_rename_field (rec_rset_t rset,
                   rec_fex_destroy (fex);
                 }
             }
-          else if (rec_field_name_eql_p (rec_field_name (field), key_field_name)
-                   || rec_field_name_eql_p (rec_field_name (field), mandatory_field_name)
-                   || rec_field_name_eql_p (rec_field_name (field), unique_field_name)
-                   || rec_field_name_eql_p (rec_field_name (field), prohibit_field_name)
+          else if (rec_field_name_eql_p (rec_field_name (field), FNAME(REC_FIELD_KEY))
+                   || rec_field_name_eql_p (rec_field_name (field), FNAME(REC_FIELD_MANDATORY))
+                   || rec_field_name_eql_p (rec_field_name (field), FNAME(REC_FIELD_UNIQUE))
+                   || rec_field_name_eql_p (rec_field_name (field), FNAME(REC_FIELD_PROHIBIT))
 #if defined REC_CRYPT_SUPPORT
-                   || rec_field_name_eql_p (rec_field_name (field), confidential_field_name)
+                   || rec_field_name_eql_p (rec_field_name (field), FNAME(REC_FIELD_CONFIDENTIAL))
 #endif
-                   || rec_field_name_eql_p (rec_field_name (field), sort_field_name))
+                   || rec_field_name_eql_p (rec_field_name (field), FNAME(REC_FIELD_SORT)))
             {
               /* Rename the field in the fex expression that is the
                  value of the field.  Skip invalid entries.  */
@@ -844,23 +488,14 @@ rec_rset_rename_field (rec_rset_t rset,
                   rec_field_set_value (field, fex_str);
                   free (fex_str);
                 }
-            }
+            }          
         }
+
+      rec_mset_iterator_free (&iter);
     }
 
   /* Update the types registry.  */
   rec_rset_update_field_props (rset);
-
-  /* Cleanup.  */
-  rec_field_name_destroy (type_field_name);
-  rec_field_name_destroy (key_field_name);
-  rec_field_name_destroy (mandatory_field_name);
-  rec_field_name_destroy (unique_field_name);
-  rec_field_name_destroy (prohibit_field_name);
-  rec_field_name_destroy (sort_field_name);
-#if defined REC_CRYPT_SUPPORT
-  rec_field_name_destroy (confidential_field_name);
-#endif
 }
 
 rec_fex_t
@@ -977,28 +612,19 @@ char *
 rec_rset_source (rec_rset_t rset)
 {
   rec_record_t record;
+
+  /* The source of the record set is considered to be the source of
+     its first record: either the descriptor or some other record.  */
   
   record = rec_rset_descriptor (rset);
   if (!record)
     {
-      record = rec_rset_elem_record (rec_rset_get_record (rset, 0));
+      record = (rec_record_t) rec_mset_get_at (rset->mset, MSET_RECORD, 0);
     }
 
   return rec_record_source (record);
 }
 
-void
-rec_rset_set_ordered (rec_rset_t rset,
-                      bool ordered_p)
-{
-  rset->ordered_p = ordered_p;
-}
-
-bool
-rec_rset_ordered (rec_rset_t rset)
-{
-  return rset->ordered_p;
-}
 
 void
 rec_rset_set_order_by_field (rec_rset_t rset,
@@ -1018,6 +644,81 @@ rec_rset_order_by_field (rec_rset_t rset)
   return rset->order_by_field;
 }
 
+void
+rec_rset_sort (rec_rset_t rset,
+               rec_field_name_t sort_by)
+{
+  if (sort_by)
+    {
+      rec_rset_set_order_by_field (rset, sort_by);
+    }
+
+  if (rset->order_by_field)
+    {
+      /* Duplicate the multi-set indicating that the elements must be
+         sorted.  */
+
+      rec_mset_sort (rset->mset);
+
+      /* Update field properties, in case order_by_field was changed
+         above.  */
+  
+      rec_rset_update_field_props (rset);
+    }
+}
+
+void
+rec_rset_add_auto_fields (rec_rset_t rset,
+                          rec_record_t record)
+{
+  rec_fex_t auto_fields;
+  rec_type_t type;
+  size_t i;
+
+  if ((auto_fields = rec_rset_auto (rset)))
+    {
+      size_t num_auto_fields = rec_fex_size (auto_fields);
+
+      for (i = 0; i < num_auto_fields; i++)
+        {
+          rec_field_name_t auto_field_name =
+            rec_fex_elem_field_name (rec_fex_get (auto_fields, i));
+
+          if (!rec_record_field_p (record, auto_field_name))
+            {
+              /* The auto field is not already present in record, so
+                 add one automatically.  Depending on its type the
+                 value is calculated differently.  If the record does
+                 not have a type, or the type is incorrect, ignore
+                 it.  */
+              
+              type = rec_rset_get_field_type (rset, auto_field_name);
+              if (type)
+                {
+                  switch (rec_type_kind (type))
+                    {
+                    case REC_TYPE_INT:
+                    case REC_TYPE_RANGE:
+                      {
+                        rec_rset_add_auto_field_int (rset, auto_field_name, record);
+                        break;
+                      }
+                    case REC_TYPE_DATE:
+                      {
+                        rec_rset_add_auto_field_date (rset, auto_field_name, record);
+                        break;
+                      }
+                    default:
+                      {
+                        /* Do nothing for other types.  */
+                        break;
+                      }
+                    }
+                }
+            }
+        }
+    }
+}
 
 /*
  * Private functions
@@ -1026,10 +727,8 @@ rec_rset_order_by_field (rec_rset_t rset)
 static void
 rec_rset_record_disp_fn (void *data)
 {
-  rec_rset_record_t rset_record = (rec_rset_record_t) data;
-
-  rec_record_destroy (rset_record->record);
-  free (rset_record);
+  rec_record_t record = (rec_record_t) data;
+  rec_record_destroy (record);
 }
 
 static bool
@@ -1044,12 +743,8 @@ rec_rset_record_equal_fn (void *data1,
 static void *
 rec_rset_record_dup_fn (void *data)
 {
-  rec_rset_record_t rset_record = (rec_rset_record_t) data;
-  rec_rset_record_t new = NULL;
-
-  new = malloc (sizeof (struct rec_rset_record_s));
-  new->rset = rset_record->rset;
-  new->record = rec_record_dup (rset_record->record);
+  rec_record_t record = (rec_record_t) data;
+  rec_record_t new = rec_record_dup (record);
 
   return (void *) new;
 }
@@ -1060,8 +755,6 @@ rec_rset_record_compare_fn (void *data1,
                             int type2)
 {
   rec_rset_t rset                  = NULL;
-  rec_rset_record_t rset_record_1  = NULL; 
-  rec_rset_record_t rset_record_2  = NULL;
   rec_field_name_t order_by_field  = NULL;
   rec_record_t record1             = NULL;
   rec_record_t record2             = NULL;
@@ -1102,15 +795,13 @@ rec_rset_record_compare_fn (void *data1,
   */
 
   /* Get the records and the containing rset.  */
-  rset_record_1 = (rec_rset_record_t) data1;
-  rset_record_2 = (rec_rset_record_t) data2;
-  record1 = rset_record_1->record;
-  record2 = rset_record_2->record;
-  rset = rset_record_1->rset;
+  record1 = (rec_record_t) data1;
+  record2 = (rec_record_t) data2;
+  rset = (rec_rset_t) rec_record_container (record1);
 
   /* Get the order by field and check if it is present in both
      registers.  */
-  order_by_field = rset_record_1->rset->order_by_field;
+  order_by_field = rset->order_by_field;
   field1 = rec_record_get_field_by_name (record1, order_by_field, 0);
   field2 = rec_record_get_field_by_name (record2, order_by_field, 0);
   
@@ -1125,7 +816,7 @@ rec_rset_record_compare_fn (void *data1,
     }
   else if (!field1 && !field2)
     {
-      return 0; /* field1 == field2 */
+      return -1; /* field1 < field2 */
     }
 
   /* Discriminate by field type.  */
@@ -1270,10 +961,8 @@ rec_rset_record_compare_fn (void *data1,
 static void
 rec_rset_comment_disp_fn (void *data)
 {
-  rec_rset_comment_t rset_comment = (rec_rset_comment_t) data;
-
-  rec_comment_destroy (rset_comment->comment);
-  free (rset_comment);
+  rec_comment_t comment = (rec_comment_t) data;
+  rec_comment_destroy (comment);
 }
 
 static bool
@@ -1288,13 +977,8 @@ rec_rset_comment_equal_fn (void *data1,
 static void *
 rec_rset_comment_dup_fn (void *data)
 {
-  rec_rset_comment_t rset_comment = (rec_rset_comment_t) data;
-  rec_rset_comment_t new = NULL;
-
-  new = malloc (sizeof (struct rec_rset_comment_s));
-  new->rset = rset_comment->rset;
-  new->comment = rec_comment_dup (rset_comment->comment);
-
+  rec_comment_t comment = (rec_comment_t) data;
+  rec_comment_t new = rec_comment_dup (comment);
   return (void *) new;
 }
 
@@ -1312,7 +996,6 @@ static void
 rec_rset_update_size_constraints (rec_rset_t rset)
 {
   rec_field_t field;
-  rec_field_name_t size_fname;
   enum rec_size_condition_e condition;
   size_t size = 0;
 
@@ -1324,10 +1007,8 @@ rec_rset_update_size_constraints (rec_rset_t rset)
      new list.  */
   if (rset->descriptor)
     {
-      size_fname = rec_parse_field_name_str ("%size:");
-
       field = rec_record_get_field_by_name (rset->descriptor,
-                                            size_fname,
+                                            FNAME(REC_FIELD_SIZE),
                                             0);
 
       if (field && rec_match (rec_field_value (field), REC_INT_SIZE_RE))
@@ -1376,21 +1057,10 @@ static void
 rec_rset_update_field_props (rec_rset_t rset)
 {
   rec_rset_fprops_t props = NULL;
-  rec_record_elem_t record_elem;
-  rec_field_t field;
-  rec_field_name_t field_name;
-  rec_field_name_t auto_fname;
   rec_field_name_t auto_field_name;
 #if defined REC_CRYPT_SUPPORT
   rec_field_name_t confidential_field_name;
-  rec_field_name_t confidential_fname;
 #endif
-  rec_field_name_t type_fname;
-  rec_field_name_t sort_fname;
-  char *field_value;
-  rec_fex_t fex;
-  size_t i;
-  rec_type_t type;
   char *type_name = NULL;
 
   /* Reset the field properties.  */
@@ -1407,34 +1077,34 @@ rec_rset_update_field_props (rec_rset_t rset)
       props = props->next;
     }
 
-  /* Scan the record descriptor for % directives, and update the
-     fields properties accordingly.  */
   if (rset->descriptor)
     {
-      auto_fname = rec_parse_field_name_str ("%auto:");
-      type_fname = rec_parse_field_name_str ("%type:");
-      sort_fname = rec_parse_field_name_str ("%sort:");
-#if defined REC_CRYPT_SUPPORT
-      confidential_fname = rec_parse_field_name_str ("%confidential:");
-#endif
+      /* Pass 1: scan the record descriptor for % directives, and update
+         the fields properties accordingly.  */
 
-      record_elem = rec_record_first_field (rset->descriptor);
-      while (rec_record_elem_p (record_elem))
+      rec_field_t field;
+      rec_mset_iterator_t iter;
+
+      iter = rec_mset_iterator (rec_record_mset (rset->descriptor));
+      while (rec_mset_iterator_next (&iter, MSET_FIELD, (const void**) &field, NULL))
         {
-          field = rec_record_elem_field (record_elem);
-          field_name = rec_field_name (field);
-          field_value = rec_field_value (field);
+          rec_field_name_t field_name = rec_field_name (field);
+          char *field_value = rec_field_value (field);
 
           /* Update field types.  Only valid %type: descriptors are
              considered.  Invalid descriptors are ignored.  */
-          if (rec_field_name_equal_p (field_name, type_fname)
+
+          if (rec_field_name_equal_p (field_name, FNAME(REC_FIELD_TYPE))
               && rec_rset_type_field_p (field_value))
             {
-              fex = rec_rset_type_field_fex (field_value);
+              size_t i;
+              rec_fex_t fex = rec_rset_type_field_fex (field_value);
+
               for (i = 0; i < rec_fex_size (fex); i++)
                 {
                   char *field_type = rec_rset_type_field_type (field_value);
-                  type = rec_type_new (field_type);
+                  rec_type_t type = rec_type_new (field_type);
+
                   if (!type)
                     {
                       /* Set field_type as a field property.  Note
@@ -1479,13 +1149,16 @@ rec_rset_update_field_props (rec_rset_t rset)
             }
 
           /* Update auto fields.  */
-          if (rec_field_name_equal_p (field_name, auto_fname))
+          if (rec_field_name_equal_p (field_name, FNAME(REC_FIELD_AUTO)))
             {
               /* %auto: fields containing incorrect data are
                   ignored.  */
-              fex = rec_fex_new (rec_field_value (field), REC_FEX_SIMPLE);
+
+              rec_fex_t fex = rec_fex_new (rec_field_value (field), REC_FEX_SIMPLE);
               if (fex)
                 {
+                  size_t i;
+
                   for (i = 0; i < rec_fex_size (fex); i++)
                     {
                       auto_field_name = rec_fex_elem_field_name (rec_fex_get (fex, i));
@@ -1498,7 +1171,7 @@ rec_rset_update_field_props (rec_rset_t rset)
           /* Update sort fields.  Since only one field can be set as
              the sorting criteria, the last field takes
              precedence.  */
-          if (rec_field_name_equal_p (field_name, sort_fname))
+          if (rec_field_name_equal_p (field_name, FNAME(REC_FIELD_SORT)))
             {
               /* Parse the field name in the field value.  Invalid
                  entries are just ignored.  */
@@ -1522,13 +1195,16 @@ rec_rset_update_field_props (rec_rset_t rset)
 
 #if defined REC_CRYPT_SUPPORT
           /* Update confidential fields.  */
-          if (rec_field_name_equal_p (field_name, confidential_fname))
+          if (rec_field_name_equal_p (field_name, FNAME(REC_FIELD_CONFIDENTIAL)))
             {
               /* Parse the field names in the field value.  Ignore
                  invalid entries.  */
-              fex = rec_fex_new (rec_field_value (field), REC_FEX_SIMPLE);
+
+              rec_fex_t fex = rec_fex_new (rec_field_value (field), REC_FEX_SIMPLE);
               if (fex)
                 {
+                  size_t i;
+
                   for (i = 0; i < rec_fex_size (fex); i++)
                     {
                       confidential_field_name =
@@ -1540,26 +1216,33 @@ rec_rset_update_field_props (rec_rset_t rset)
             }
 #endif /* REC_CRYPT_SUPPORT */
 
-          record_elem = rec_record_next_field (rset->descriptor, record_elem);
         }
 
-      rec_field_name_destroy (auto_fname);
-      rec_field_name_destroy (type_fname);
-      rec_field_name_destroy (sort_fname);
-#if defined REC_CRYPT_SUPPORT
-      rec_field_name_destroy (confidential_fname);
-#endif
-    } 
+      rec_mset_iterator_free (&iter);
+    }
+
+  /* Pass 2: scan the fields having properties on the record set.  */
+
+  props = rset->field_props;
+  while (props)
+    {
+      /* Auto fields not having an explicit type are implicitly
+         typed as integers.  */
+      
+      if (props->auto_p && !props->type && !props->type_name)
+        {
+          props->type = rec_type_new ("int");
+        }
+      
+      props = props->next;
+    }
 }
 
 static void
 rec_rset_update_types (rec_rset_t rset)
 {
   rec_field_t field;
-  rec_record_elem_t record_elem;
-  rec_field_name_t field_name;
-  char *field_value;
-  rec_field_name_t typedef_fname;
+  rec_mset_iterator_t iter;
   const char *p, *q = NULL;
   rec_type_t type;
   char *type_name, *to_type = NULL;
@@ -1569,22 +1252,20 @@ rec_rset_update_types (rec_rset_t rset)
      types registry accordingly.  */
   if (rset->descriptor)
     {
-      /* Create some field names.  */
-      typedef_fname = rec_parse_field_name_str ("%typedef:");
-
       /* Purge the registry.  */
+
       rec_type_reg_destroy (rset->type_reg);
       rset->type_reg = rec_type_reg_new ();
 
       /* Iterate on the fields of the descriptor.  */
-      record_elem = rec_record_first_field (rset->descriptor);
-      while (rec_record_elem_p (record_elem))
-        {
-          field = rec_record_elem_field (record_elem);
-          field_name = rec_field_name (field);
-          field_value = rec_field_value (field);
 
-          if (rec_field_name_equal_p (field_name, typedef_fname))
+      iter = rec_mset_iterator (rec_record_mset (rset->descriptor));
+      while (rec_mset_iterator_next (&iter, MSET_FIELD, (const void **) &field, NULL))
+        {
+          rec_field_name_t field_name = rec_field_name (field);
+          char *field_value = rec_field_value (field);
+
+          if (rec_field_name_equal_p (field_name, FNAME(REC_FIELD_TYPEDEF)))
             {
               p = field_value;
               rec_skip_blanks (&p);
@@ -1624,12 +1305,9 @@ rec_rset_update_types (rec_rset_t rset)
                   free (type_name);
                 }
             }
-
-          record_elem = rec_record_next_field (rset->descriptor,
-                                               record_elem);
         }
 
-      rec_field_name_destroy (typedef_fname);
+      rec_mset_iterator_free (&iter);
     }
 }
 
@@ -1642,7 +1320,7 @@ rec_rset_type_field_p (const char *str)
 
   rec_skip_blanks (&p);
   if (!rec_parse_regexp (&p,
-                         "^" REC_FNAME_RE "(," REC_FNAME_RE ")*",
+                         "^" REC_FNAME_LIST_CS_RE,
                          NULL))
     {
       return false;
@@ -1665,7 +1343,7 @@ rec_rset_type_field_fex (const char *str)
   p = str;
 
   if (rec_parse_regexp (&p,
-                        "^" REC_FNAME_RE "(," REC_FNAME_RE ")*",
+                        "^" REC_FNAME_LIST_CS_RE,
                         &name))
     {
       fex = rec_fex_new (name, REC_FEX_CSV);
@@ -1686,7 +1364,7 @@ rec_rset_type_field_type (const char *str)
       p = str;
 
       rec_skip_blanks (&p);
-      rec_parse_regexp (&p, "^" REC_FNAME_RE "(," REC_FNAME_RE ")*", NULL);
+      rec_parse_regexp (&p, "^" REC_FNAME_LIST_CS_RE, NULL);
       rec_skip_blanks (&p);
 
       /* Return the rest of the string.  */
@@ -1740,5 +1418,78 @@ rec_rset_get_props (rec_rset_t rset,
   return props;
 }
 
+static void
+rec_rset_add_auto_field_int (rec_rset_t rset,
+                             rec_field_name_t field_name,
+                             rec_record_t record)
+{
+  rec_mset_iterator_t iter;
+  rec_record_t rec;
+  rec_field_t field;
+  size_t num_fields, i;
+  int auto_value, field_value;
+  char *end;
+  char *auto_value_str;
+
+  /* Find the auto value.  */
+
+  auto_value = 0;
+
+  iter = rec_mset_iterator (rec_rset_mset (rset));
+  while (rec_mset_iterator_next (&iter, MSET_RECORD, (const void **) &rec, NULL))
+    {
+      num_fields = rec_record_get_num_fields_by_name (rec, field_name);
+      for (i = 0; i < num_fields; i++)
+        {
+          field = rec_record_get_field_by_name (rec, field_name, i);
+          
+          /* Ignore fields that can't be converted to integer
+             values.  */
+          errno = 0;
+          field_value = strtol (rec_field_value (field), &end, 10);
+          if ((errno == 0) && (*end == '\0'))
+            {
+              if (auto_value <= field_value)
+                {
+                  auto_value = field_value + 1;
+                }
+            }
+        }
+    }
+
+  rec_mset_iterator_free (&iter);
+       
+  /* Create and insert the auto field.  */
+
+  asprintf (&auto_value_str, "%d", auto_value);
+  field = rec_field_new (rec_field_name_dup (field_name),
+                         auto_value_str);
+  rec_mset_insert_at (rec_record_mset (record), MSET_FIELD, (void *) field, 0);
+  free (auto_value_str);
+}
+
+static void
+rec_rset_add_auto_field_date (rec_rset_t rset,
+                              rec_field_name_t field_name,
+                              rec_record_t record)
+{
+  rec_field_t auto_field;
+  time_t t;
+  char outstr[200];
+  struct tm *tmp;
+
+  t = time (NULL);
+  tmp = localtime (&t);
+
+  setlocale (LC_TIME, "C"); /* We want english dates that can be
+                                 parsed with parse_datetime */
+  strftime (outstr, sizeof(outstr), "%a, %d %b %Y %T %z", tmp);
+  setlocale (LC_TIME, ""); /* And restore the locale from the
+                              environment. */
+
+  auto_field = rec_field_new (rec_field_name_dup (field_name),
+                              outstr);
+  rec_mset_insert_at (rec_record_mset (record), MSET_FIELD, (void *) auto_field, 0);
+}
 
 /* End of rec-rset.c */
