@@ -55,8 +55,8 @@
 #define REC_TYPE_EMAIL_NAME  "email"
 #define REC_TYPE_FIELD_NAME  "field"
 
-/* Regular expression denoting a blank character in a type
-   description.  */
+/* Regular expressions.  */
+
 #define REC_TYPE_BLANK_RE "[ \t\n]"
 #define REC_TYPE_NO_BLANK_RE "[^ \t\n]"
 #define REC_TYPE_BLANKS_RE REC_TYPE_BLANK_RE "+"
@@ -65,7 +65,7 @@
 
 /* Regular expressions denoting values.  */
 #define REC_TYPE_INT_VALUE_RE                   \
-  "^" REC_TYPE_ZBLANKS_RE "-?[0-9]+" REC_TYPE_ZBLANKS_RE "$"
+  "^" REC_TYPE_ZBLANKS_RE REC_INT_RE REC_TYPE_ZBLANKS_RE "$"
 
 #define REC_TYPE_BOOL_VALUE_RE                  \
   "^" REC_TYPE_ZBLANKS_RE "(yes|no|true|false|0|1)" REC_TYPE_ZBLANKS_RE "$"
@@ -122,10 +122,10 @@
 #define REC_TYPE_RANGE_DESCR_RE                    \
   REC_TYPE_RANGE_NAME                              \
   REC_TYPE_BLANKS_RE                               \
-  "-?[0-9]+"                                       \
+  REC_INT_RE                                       \
   "("                                              \
   REC_TYPE_ZBLANKS_RE                              \
-  "-?[0-9]+"                                       \
+  REC_INT_RE                                       \
   ")?"
 
 /* real  */
@@ -136,7 +136,7 @@
 #define REC_TYPE_SIZE_DESCR_RE                     \
   REC_TYPE_SIZE_NAME                               \
   REC_TYPE_BLANKS_RE                               \
-  "[0-9]+"
+  REC_INT_RE
 
 /* line  */
 #define REC_TYPE_LINE_DESCR_RE                  \
@@ -232,7 +232,6 @@ struct rec_type_reg_s
  * Forward declarations.
  */
 
-static bool rec_type_check_re (char *regexp_str, char *str);
 static enum rec_type_kind_e rec_type_parse_type_kind (char *str);
 
 static bool rec_type_check_int (rec_type_t type, char *str, rec_buf_t errors);
@@ -846,27 +845,6 @@ rec_type_max (rec_type_t type)
  * Private functions.
  */
 
-static bool
-rec_type_check_re (char *regexp_str,
-                   char *str)
-{
-  int status;
-  regex_t regexp;
-
-  /* Compile the regexp.  */
-  if ((status = regcomp (&regexp, regexp_str, REG_EXTENDED)) != 0)
-    {
-      fprintf (stderr, _("internal error: rec-types: error compiling regexp.\n"));
-      printf("YYY: %s\n", regexp_str);
-      return false;
-    }
-
-  /* Check.  */
-  status = regexec (&regexp, str, 0, NULL, 0);
-
-  return (status == 0);
-}
-
 static enum rec_type_kind_e
 rec_type_parse_type_kind (char *str)
 {
@@ -929,7 +907,7 @@ rec_type_check_int (rec_type_t type,
 {
   bool ret;
 
-  ret = rec_type_check_re (REC_TYPE_INT_VALUE_RE, str);
+  ret = rec_match (str, REC_TYPE_INT_VALUE_RE);
   if (!ret && errors)
     {
       rec_buf_puts (_("invalid integer."), errors);
@@ -945,7 +923,7 @@ rec_type_check_field (rec_type_t type,
 {
   bool ret;
 
-  ret = rec_type_check_re (REC_TYPE_FIELD_VALUE_RE, str);
+  ret = rec_match (str, REC_TYPE_FIELD_VALUE_RE);
   if (!ret && errors)
     {
       rec_buf_puts (_("invalid 'field' value."), errors);
@@ -961,7 +939,7 @@ rec_type_check_bool (rec_type_t type,
 {
   bool ret;
 
-  ret = rec_type_check_re (REC_TYPE_BOOL_VALUE_RE, str);
+  ret = rec_match (str, REC_TYPE_BOOL_VALUE_RE);
   if (!ret && errors)
     {
       rec_buf_puts (_("invalid 'bool' value."), errors);
@@ -1012,7 +990,7 @@ rec_type_check_real (rec_type_t type,
 {
   bool ret;
 
-  ret = rec_type_check_re (REC_TYPE_REAL_VALUE_RE, str);
+  ret = rec_match (str, REC_TYPE_REAL_VALUE_RE);
   if (!ret && errors)
     {
       rec_buf_puts (_("invalid 'real' value."), errors);
@@ -1049,7 +1027,7 @@ rec_type_check_line (rec_type_t type,
 {
   bool ret;
 
-  ret = rec_type_check_re (REC_TYPE_LINE_VALUE_RE, str);
+  ret = rec_match (str, REC_TYPE_LINE_VALUE_RE);
   if (!ret && errors)
     {
       rec_buf_puts (_("invalid 'line' value."), errors);
@@ -1107,7 +1085,7 @@ rec_type_check_email (rec_type_t type,
 {
   bool ret;
 
-  ret = rec_type_check_re (REC_TYPE_EMAIL_VALUE_RE, str);
+  ret = rec_match (str, REC_TYPE_EMAIL_VALUE_RE);
   if (!ret && errors)
     {
       rec_buf_puts (_("invalid email."), errors);
@@ -1125,7 +1103,7 @@ rec_type_check_enum (rec_type_t type,
   const char *p, *b;
   char name[100];
 
-  if (!rec_type_check_re (REC_TYPE_ENUM_VALUE_RE, str))
+  if (!rec_match (str, REC_TYPE_ENUM_VALUE_RE))
     {
       return false;
     }
@@ -1182,7 +1160,8 @@ rec_type_parse_size (const char *str, rec_type_t type)
   rec_skip_blanks (&p);
 
   /* Get the size.  */
-  if (rec_parse_int (&p, &size))
+  if (rec_parse_int (&p, &size)
+      && (size >= 0))
     {
       type->data.max_size = size;
     }
