@@ -7,7 +7,7 @@
  *
  */
 
-/* Copyright (C) 2009, 2010, 2011 Jose E. Marchesi */
+/* Copyright (C) 2009, 2010, 2011, 2012 Jose E. Marchesi */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -31,8 +31,9 @@
 #include <xalloc.h>
 #include <gettext.h>
 #include <errno.h>
-#include <locale.h>
 #include <time.h>
+#include <locale.h>
+#include <getpass.h>
 #define _(str) gettext (str)
 #include <base64.h>
 
@@ -50,7 +51,7 @@ void recins_parse_args (int argc, char **argv);
 char         *recutl_type      = NULL;
 rec_sex_t     recutl_sex       = NULL;
 char         *recutl_sex_str   = NULL;
-int           recutl_num       = -1;
+char         *recutl_quick_str = NULL;
 bool          recutl_insensitive = false;
 rec_record_t  recins_record    = NULL;
 char         *recins_file      = NULL;
@@ -59,6 +60,7 @@ bool          recins_verbose   = false;
 bool          recins_external  = true;
 bool          recins_auto      = true;
 char         *recins_password  = NULL;
+size_t        recutl_random    = 0;
 
 /*
  * Command line options management
@@ -104,7 +106,7 @@ recutl_print_help (void)
   /* TRANSLATORS: --help output, recins synopsis.
      no-wrap */
   printf (_("\
-Usage: recins [OPTION]... [t TYPE] [-n NUM | -e EXPR] [(-f STR -v STR]|[-r RECDATA)]... [FILE]\n"));
+Usage: recins [OPTION]... [t TYPE] [-n NUM | -e EXPR | -q STR | -m NUM] [(-f STR -v STR]|[-R RECDATA)]... [FILE]\n"));
 
   /* TRANSLATORS: --help output, recins short description.
      no-wrap */
@@ -149,152 +151,36 @@ the data from standard input and writing the result to standard output.\n"), std
   recutl_print_help_footer ();
 }
 
+#if defined REC_CRYPT_SUPPORT
+
 void
-recins_add_auto_field_int (rec_rset_t rset,
-                           rec_field_name_t field_name,
-                           rec_record_t record)
+recins_encrypt_record (rec_rset_t rset,
+                       rec_record_t record)
 {
-  rec_rset_elem_t rset_elem;
-  rec_record_t rec;
-  rec_field_t field;
-  size_t num_fields, i;
-  int auto_value, field_value;
-  char *end;
-  char *auto_value_str;
-  rec_record_elem_t rec_elem;
+  /* Encrypt the value of fields declared as confidential in this
+     record set.  */
 
-  /* Find the auto value.  */
+  rec_fex_t confidential_fields = rec_rset_confidential (rset);
 
-  auto_value = 0;
-
-  rset_elem = rec_rset_first_record (rset);
-  while (rec_rset_elem_p (rset_elem))
+  if (rec_fex_size (confidential_fields) > 0)
     {
-      rec = rec_rset_elem_record (rset_elem);
-
-      num_fields = rec_record_get_num_fields_by_name (rec, field_name);
-      for (i = 0; i < num_fields; i++)
+      if (!recins_password && recutl_interactive ())
         {
-          field = rec_record_get_field_by_name (rec, field_name, i);
-          
-          /* Ignore fields that can't be converted to integer
-             values.  */
-          errno = 0;
-          field_value = strtol (rec_field_value (field), &end, 10);
-          if ((errno == 0) && (*end == '\0'))
-            {
-              if (auto_value <= field_value)
-                {
-                  auto_value = field_value + 1;
-                }
-            }
+          recins_password = getpass (_("Password: "));
         }
-
-      rset_elem = rec_rset_next_record (rset, rset_elem);
-    }
-       
-  /* Insert the auto field.  */
-  asprintf (&auto_value_str, "%d", auto_value);
-  field = rec_field_new (rec_field_name_dup (field_name),
-                         auto_value_str);
-  rec_elem = rec_record_elem_field_new (record, field);
-  rec_record_insert_at (record, rec_elem, 0);
-  free (auto_value_str);
-}
-
-void
-recins_add_auto_field_date (rec_rset_t rset,
-                            rec_field_name_t field_name,
-                            rec_record_t record)
-{
-  rec_field_t auto_field;
-  rec_record_elem_t rec_elem;
-  time_t t;
-  char outstr[200];
-  struct tm *tmp;
-
-  t = time (NULL);
-  tmp = localtime (&t);
-
-  setlocale (LC_TIME, "C"); /* We want english dates that can be
-                                 parsed with parse_datetime */
-  strftime (outstr, sizeof(outstr), "%a, %d %b %Y %T %z", tmp);
-  setlocale (LC_TIME, ""); /* And restore the locale from the
-                              environment. */
-
-  auto_field = rec_field_new (rec_field_name_dup (field_name),
-                              outstr);
-  rec_elem = rec_record_elem_field_new (record, auto_field);
-  rec_record_insert_at (record, rec_elem, 0);
-}
-
-rec_record_t
-recins_add_auto_fields (rec_rset_t rset,
-                        rec_record_t record)
-{
-  rec_record_t res = NULL;
-  rec_field_name_t auto_field_name;
-  rec_fex_t auto_fields;
-  rec_type_t type;
-  int i;
-
-  res = rec_record_dup (record);
-
-  if (!recins_auto)
-    {
-      return res;
-    }
-
-  if (rset)
-    {
-      if ((auto_fields = rec_rset_auto (rset)))
+      
+      if (recins_password && (strlen (recins_password) > 0))
         {
-          for (i = 0; i < rec_fex_size (auto_fields); i++)
+          if (!rec_encrypt_record (rset, record, recins_password))
             {
-              auto_field_name = rec_fex_elem_field_name (rec_fex_get (auto_fields, i));
-              
-              if (!rec_record_field_p (record, auto_field_name))
-                {
-                  /* The auto field is not already present in record, so add
-                     one automatically.  Depending on its type the value is
-                     calculated differently. If the record does not have a
-                     type, or the type is incorrect, ignore it.  */
-                  
-                  type = rec_rset_get_field_type (rset, auto_field_name);
-                  if (type)
-                    {
-                      switch (rec_type_kind (type))
-                        {
-                        case REC_TYPE_INT:
-                        case REC_TYPE_RANGE:
-                          {
-                            recins_add_auto_field_int (rset,
-                                                       auto_field_name,
-                                                       res);
-                            break;
-                          }
-                        case REC_TYPE_DATE:
-                          {
-                            recins_add_auto_field_date (rset,
-                                                        auto_field_name,
-                                                        res);
-                            break;
-                          }
-                        default:
-                          {
-                            /* Do nothing for other types.  */
-                            break;
-                          }
-                        }
-                    }
-                }
+              recutl_fatal ("encrypting a record.  Please report this.\n");
             }
         }
     }
-
-  return res;
 }
 
+#endif /* REC_CRYPT_SUPPORT */
+
 bool
 recins_insert_record (rec_db_t db,
                       char *type,
@@ -302,8 +188,6 @@ recins_insert_record (rec_db_t db,
 {
   bool res;
   rec_rset_t rset;
-  rec_rset_elem_t last_elem, new_elem;
-  rec_record_t record_to_insert;
 
   if (!record || (rec_record_num_fields (record) == 0))
     {
@@ -316,33 +200,19 @@ recins_insert_record (rec_db_t db,
   rset = rec_db_get_rset_by_type (db, type);
   if (rset)
     {
-      /* Add auto-set fields required by this record set.  */
-      record_to_insert = recins_add_auto_fields (rset, record);
-      rec_record_destroy (record);
+
+      if (recins_auto)
+        {
+          /* Add auto-set fields required by this record set.  */
+          rec_rset_add_auto_fields (rset, record);
+        }
 
 #if defined REC_CRYPT_SUPPORT
-      {
-        /* Encrypt the value of fields declared as confidential in
-           this record set.  */
-        
-        rec_fex_t confidential_fields =
-          rec_rset_confidential (rset);
-        if (!recins_password && (rec_fex_size (confidential_fields) > 0))
-          {
-            recutl_warning (_("the record set contains confidential fields but no password was provided\n"));
-            recutl_warning (_("the resulting record will have those fields unencrypted!\n"));
-          }
-        else
-          {
-            if (!rec_encrypt_record (rset, record_to_insert, recins_password))
-              {
-                recutl_fatal ("encrypting a record.  Please report this.\n");
-              }
-          }
-      }
+      recins_encrypt_record (rset, record);
 #endif
 
-      new_elem = rec_rset_elem_record_new (rset, record_to_insert);
+      /* Set the container in the new record.  */
+      rec_record_set_container (record, rset);
 
       if (rec_rset_num_records (rset) == 0)
         {
@@ -351,15 +221,27 @@ recins_insert_record (rec_db_t db,
 
              XXX: move this logic into an 'rec_rset_prepend_record' function.
           */
-          rec_rset_insert_at (rset, new_elem, rec_rset_descriptor_pos (rset));
+
+          rec_mset_insert_at (rec_rset_mset (rset),
+                              MSET_RECORD,
+                              (void *) record,
+                              rec_rset_descriptor_pos (rset));
         }
       else
         {
           /* Insert the new record after the last record in the
              set.  */
-          last_elem = rec_rset_get_record (rset,
-                                           rec_rset_num_records (rset) - 1);
-          rec_rset_insert_after (rset, last_elem, new_elem);
+
+          rec_mset_t mset = rec_rset_mset (rset);
+          rec_record_t last_record =
+            (rec_record_t) rec_mset_get_at (mset,
+                                            MSET_RECORD,
+                                            rec_rset_num_records (rset) - 1);
+
+          rec_mset_insert_after (mset,
+                                 MSET_RECORD,
+                                 (void *) record,
+                                 rec_mset_search (mset, (void *) last_record));
         }
     }
   else
@@ -368,7 +250,8 @@ recins_insert_record (rec_db_t db,
          to check for the type of the field in this case.  */
       rset = rec_rset_new ();
       rec_rset_set_type (rset, type);
-      rec_rset_append_record (rset, record);
+      rec_record_set_container (record, rset);
+      rec_mset_append (rec_rset_mset (rset), MSET_RECORD, (void *) record, MSET_ANY);
       
       if (type)
         {
@@ -394,7 +277,6 @@ void recins_parse_args (int argc,
   rec_field_name_t field_name;
   char *field_name_str;
   rec_record_t provided_record;
-  rec_record_elem_t rec_elem;
 
   field = NULL;
   field_name_str = NULL;
@@ -468,7 +350,7 @@ void recins_parse_args (int argc,
               }
 
             rec_field_set_value (field, optarg);
-            rec_record_append_field (recins_record, field);
+            rec_mset_append (rec_record_mset (recins_record), MSET_FIELD, (void *) field, MSET_ANY);
 
             field = NULL;
             break;
@@ -508,14 +390,14 @@ void recins_parse_args (int argc,
               {
                 /* Append the fields in provided_record into
                    recins_record.  */
-                rec_elem = rec_record_first_field (provided_record);
-                while (rec_record_elem_p (rec_elem))
+                
+                rec_mset_iterator_t iter = rec_mset_iterator (rec_record_mset (provided_record));
+                while (rec_mset_iterator_next (&iter, MSET_FIELD, (const void **) &field, NULL))
                   {
-                    field = rec_record_elem_field (rec_elem);
-                    rec_record_append_field (recins_record, rec_field_dup (field));
+                    rec_mset_append (rec_record_mset (recins_record), MSET_FIELD, (void *) rec_field_dup (field), MSET_ANY);
                     field = NULL;
-                    rec_elem = rec_record_next_field (provided_record, rec_elem);
                   }
+                rec_mset_iterator_free (&iter);
 
                 rec_record_destroy (provided_record);
                 provided_record = NULL;
@@ -558,55 +440,52 @@ recins_add_new_record (rec_db_t db)
 {
   rec_rset_t rset;
   rec_record_t record;
-  rec_rset_elem_t rset_elem;
-  rec_rset_elem_t new_rset_elem;
+  rec_mset_iterator_t iter;
+  rec_mset_elem_t elem;
   size_t num_rec;
   bool parse_status;
-  rec_record_t record_to_insert;
 
-  if ((recutl_num != -1)
-      || (recutl_sex_str != NULL))
+  if ((recutl_num_indexes() != 0)
+      || (recutl_sex_str != NULL)
+      || (recutl_quick_str != NULL)
+      || (recutl_random > 0))
     {
       /* Replace matching records.  */
       rset = rec_db_get_rset_by_type (db, recutl_type);
       if (rset)
         {
           num_rec = -1;
+
+
+          /* If the user requested to replace random records,
+             calculate them now for this record set.  */
+
+          if (recutl_random > 0)
+            {
+              recutl_reset_indexes ();
+              recutl_index_add_random (recutl_random, rec_rset_num_records (rset));
+            }
      
-          /* Add auto-set fields required by this record set.  */
-          record_to_insert = recins_add_auto_fields (rset, recins_record);
-          rec_record_destroy (recins_record);
+          if (recins_auto)
+            {
+              /* Add auto-set fields required by this record set.  */
+              rec_rset_add_auto_fields (rset, recins_record);
+            }
 
 #if defined REC_CRYPT_SUPPORT
-      {
-        /* Encrypt the value of fields declared as confidential in
-           this record set.  */
-        
-        rec_fex_t confidential_fields =
-          rec_rset_confidential (rset);
-        if (!recins_password && (rec_fex_size (confidential_fields) > 0))
-          {
-            recutl_warning (_("the record set contains confidential fields but no password was provided\n"));
-            recutl_warning (_("the resulting record will have those fields unencrypted!\n"));
-          }
-        else
-          {
-            if (!rec_encrypt_record (rset, record_to_insert, recins_password))
-              {
-                recutl_fatal ("encrypting a record.  Please report this.\n");
-              }
-          }
-      }
+          recins_encrypt_record (rset, recins_record);
 #endif
 
-          rset_elem = rec_rset_first_record (rset);
-          while (rec_rset_elem_p (rset_elem))
+          iter = rec_mset_iterator (rec_rset_mset (rset));
+          while (rec_mset_iterator_next (&iter, MSET_RECORD, (const void **) &record, &elem))
             {
               num_rec++;
-              record = rec_rset_elem_record (rset_elem);
 
-              /* Shall we skip this record?  */
-              if (((recutl_num != -1) && (recutl_num != num_rec))
+              /* Shall we skip this record  */
+              if (((recutl_num_indexes() != 0) && (!recutl_index_p (num_rec)))
+                  || (recutl_quick_str && !rec_record_contains_value (record,
+                                                                      recutl_quick_str,
+                                                                      recutl_insensitive))
                   || (recutl_sex_str && !(rec_sex_eval (recutl_sex, record, &parse_status)
                                           && parse_status)))
                 {
@@ -615,17 +494,16 @@ recins_add_new_record (rec_db_t db)
                       recutl_error (_("evaluating the selection expression.\n"));
                       exit (EXIT_FAILURE);
                     }
-                  rset_elem = rec_rset_next_record (rset, rset_elem);
                 }
               else
                 {
-                  new_rset_elem = rec_rset_elem_record_new (rset, record_to_insert);
-                  rec_rset_insert_after (rset, rset_elem, new_rset_elem);
-                  rec_rset_remove (rset, rset_elem);
-                  rset_elem = rec_rset_next_record (rset, new_rset_elem);
+                  rec_record_set_container (recins_record, rset);
+                  rec_mset_elem_set_data (elem, (void *) rec_record_dup (recins_record));
                 }
             }
         }
+
+      rec_mset_iterator_free (&iter);
     }
   else
     {
