@@ -7,7 +7,7 @@
  *
  */
 
-/* Copyright (C) 2010, 2011 Jose E. Marchesi */
+/* Copyright (C) 2010, 2011, 2012 Jose E. Marchesi */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -61,17 +61,21 @@
 #define RECORD_SELECTION_ARGS                   \
   TYPE_ARG,                                     \
   EXPRESSION_ARG,                               \
+  QUICK_ARG,                                    \
   NUM_ARG,                                      \
-  INSENSITIVE_ARG
+  INSENSITIVE_ARG,                              \
+  RANDOM_ARG
 
 #define RECORD_SELECTION_LONG_ARGS                                     \
    {"type", required_argument, NULL, TYPE_ARG},                        \
    {"expression", required_argument, NULL, EXPRESSION_ARG},            \
+   {"quick", required_argument, NULL, QUICK_ARG},                      \
    {"num", required_argument, NULL, NUM_ARG},                          \
-   {"case-insensitive", no_argument, NULL, INSENSITIVE_ARG}
+   {"case-insensitive", no_argument, NULL, INSENSITIVE_ARG},           \
+   {"random", required_argument, NULL, RANDOM_ARG}
 
 #define RECORD_SELECTION_SHORT_ARGS             \
-   "it:e:n:"
+   "it:e:n:q:m:"
 
 #define RECORD_SELECTION_ARGS_CASES                            \
     case TYPE_ARG:                                             \
@@ -88,10 +92,15 @@
     case EXPRESSION_ARG:                                       \
     case 'e':                                                  \
       {                                                        \
-        if (recutl_num != -1)                                  \
+        if (recutl_num_indexes() != 0)                         \
+          {                                                    \
+            recutl_fatal (_("cannot specify -e and also -n\n"));\
+          }                                                    \
+                                                               \
+        if (recutl_quick_str)                                  \
           {                                                    \
              fprintf (stderr,                                  \
-                      "%s: cannot specify -e and also -n.\n",  \
+                      "%s: cannot specify -e and also -q.\n",  \
                       program_name);                           \
              exit (EXIT_FAILURE);                              \
           }                                                    \
@@ -116,9 +125,6 @@
       case NUM_ARG:                                            \
       case 'n':                                                \
       {                                                        \
-         long int li;                                          \
-         char *end;                                            \
-         char *str;                                            \
                                                                \
          if (recutl_sex)                                       \
           {                                                    \
@@ -128,22 +134,89 @@
              exit (EXIT_FAILURE);                              \
           }                                                    \
                                                                \
-          str = xstrdup (optarg);                              \
-          li = strtol (str, &end, 10);                         \
-          if ((*str != '\0') && (*end == '\0'))                \
-            {                                                  \
-              /* Valid number.  */                             \
-              recutl_num = (int) li;                           \
-            }                                                  \
-          else                                                 \
-            {                                                  \
-              fprintf (stderr,                                 \
-                       "%s: invalid number '%s' in -n.\n",     \
-                       program_name, str);                     \
-              exit (EXIT_FAILURE);                             \
-            }                                                  \
+        if (recutl_quick_str)                                  \
+          {                                                    \
+             fprintf (stderr,                                  \
+                      "%s: cannot specify -n and also -q\n",   \
+                      program_name);                           \
+             exit (EXIT_FAILURE);                              \
+          }                                                    \
+                                                               \
+        if (recutl_num_indexes() != 0)                         \
+          {                                                    \
+             recutl_fatal ("please specify just one -n\n");    \
+          }                                                    \
+                                                               \
+        if (!recutl_index_list_parse (optarg))                 \
+          {                                                    \
+            recutl_fatal (_("invalid list of indexes in -n\n")); \
+          }                                                    \
+                                                               \
           break;                                               \
       }                                                        \
+      case RANDOM_ARG:                                         \
+      case 'm':                                                \
+      {                                                        \
+        if (recutl_sex)                                        \
+          {                                                    \
+             fprintf (stderr,                                  \
+                      "%s: cannot specify -m and also -e.\n",  \
+                      program_name);                           \
+             exit (EXIT_FAILURE);                              \
+          }                                                    \
+                                                               \
+        if (recutl_quick_str)                                  \
+          {                                                    \
+             fprintf (stderr,                                  \
+                      "%s: cannot specify -m and also -q\n",   \
+                      program_name);                           \
+             exit (EXIT_FAILURE);                              \
+          }                                                    \
+                                                               \
+        if (recutl_num_indexes() != 0)                         \
+          {                                                    \
+             fprintf (stderr,                                  \
+                      "%s: cannot specify -m and also -n\n",   \
+                      program_name);                           \
+             exit (EXIT_FAILURE);                              \
+          }                                                    \
+                                                               \
+        if (recutl_random)                                     \
+          {                                                    \
+             recutl_fatal ("please specify just one -m\n");    \
+          }                                                    \
+                                                               \
+        {                                                      \
+          char *end;                                           \
+          long int li = strtol (optarg, &end, 0);              \
+          if (*end != '\0')                                    \
+            {                                                  \
+              recutl_fatal ("invalid number in -m\n");         \
+            }                                                  \
+                                                               \
+          recutl_random = li;                                  \
+        }                                                      \
+                                                               \
+        break;                                                 \
+      }                                                        \
+      case QUICK_ARG:                                          \
+      case 'q':                                                \
+      {                                                        \
+         if (recutl_sex)                                       \
+          {                                                    \
+             fprintf (stderr,                                  \
+                      "%s: cannot specify -n and also -e.\n",  \
+                      program_name);                           \
+             exit (EXIT_FAILURE);                              \
+          }                                                    \
+        if (recutl_num_indexes() != 0)                         \
+          {                                                    \
+            recutl_fatal (_("cannot specify -e and also -n")); \
+          }                                                    \
+                                                               \
+        recutl_quick_str = xstrdup (optarg);                   \
+        break;                                                 \
+      }                                                        \
       case INSENSITIVE_ARG:                                    \
       case 'i':                                                \
       {                                                        \
@@ -172,7 +245,8 @@ char *recutl_eval_field_expression (rec_fex_t fex,
                                     rec_record_t record,
                                     rec_writer_mode_t mode,
                                     bool print_values_p,
-                                    bool print_in_a_row_p);
+                                    bool print_in_a_row_p,
+                                    char *password);
 
 rec_db_t recutl_read_db_from_file (char *file_name);
 void recutl_write_db_to_file (rec_db_t db, char *file_name);
@@ -183,9 +257,34 @@ void recutl_check_integrity (rec_db_t db,
                              bool verbose_p,
                              bool external_p);
 
-void recutl_sorting_parser (bool sort_p,
-                            char *rset_name,
-                            rec_field_name_t field_name);
+bool recutl_yesno (char *prompt);
+
+bool recutl_interactive (void);
+
+/* Parse a list of indexes from the given string and set the internal
+   recutl_indexes accordingly.  Return true if a list was found in the
+   string.  Return false otherwise.  */
+
+bool recutl_index_list_parse (const char *str);
+
+/* Add a given number of random indexes to the indexes list.  The
+   generated random numbers are in the range 0..LIMIT.  */
+
+void recutl_index_add_random (size_t num, size_t limit);
+
+/* Return the number of indexes in the internal recutl_indexes data
+   structure.  */
+
+size_t recutl_num_indexes (void);
+
+/* Return whether an index is stored in the internal recutl_indexes
+   data structure.  */
+
+bool recutl_index_p (size_t index);
+
+/* Reset the list of indexes to be empty.  */
+
+void recutl_reset_indexes (void);
 
 #endif /* recutl.h */
 
